﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;

namespace ComTool
{
    /*---------------作者：叶知秋----------------------*/
    /*---------------时间：2012年9月14日---------------*/
    /*---------------邮箱：yq@yyzq.net---------*/
    /*---------------yyzq.net--------------------------*/
    /*---------------tczhuoyou.com---------------------*
    /*任何个人均可免费修改，使用本程序，但请保留以上作者信息，谢谢！*/

    /// <summary>
    /// 主要实现功能：只能输入数字0~9以及小数点.
    /// 通过设置DecimalPlaces属性可输入小数点位数
    /// 通过设置MaxLength属性设置输入的数字长度
    /// 禁止粘贴数据
    /// </summary>
    public partial class NumberTextBox : TextBox
    {
        private int _DecimalPlaces;

        [Description("小数位数,0表示不能输入小数")]
        public int DecimalPlaces
        {
            set { _DecimalPlaces = value; }
            get { return _DecimalPlaces; }
        }

        public NumberTextBox()
        {
            InitializeComponent();
        }

        protected override void OnTextChanged(EventArgs e)
        {
            int Pos = this.Text.IndexOf('.');//如果小数点是第一位则删除小数点
            if (Pos == 0)
            {
                this.Text = this.Text.Remove(0, 1);
            }
            if (this.Text.Length > 1)
            {//不止一位数字
                string InputNumber = this.Text;
                for (int i = 0; i < InputNumber.Length; i++)
                {
                    if (InputNumber.Substring(i, 1) == "0")
                    { //是0
                        InputNumber = InputNumber.Remove(i, 1);//删除头部的0
                        i--;
                    }
                    else
                    {
                        break;
                    }
                }
                this.Text = InputNumber;
            }
            if (this.Text.Length == 0)
            {//默认值为0
                this.Text = "0";
            }
        }

        protected override void OnKeyPress(KeyPressEventArgs e)
        {
            if (e.KeyChar >= 0x30 && e.KeyChar <= 0x39)
            {
                if (DecimalPlaces >= 0 && this.SelectionLength == 0)
                {
                    int Pos = this.Text.IndexOf('.');
                    if (this.SelectionStart > Pos && Pos >= 0)
                    {//输入的是小数位
                        int DecimalLength = this.Text.Length - Pos - 1;//小数位数
                        if (DecimalLength >= DecimalPlaces)
                        {
                            e.Handled = true;//true表示不用处理的输入，false表示继续往下处理
                            return;
                        }
                    }
                }
                e.Handled = false;
                return;
            }
            if (e.KeyChar == 0x08)//退格
            {
                e.Handled = false;
                return;
            }
            if (e.KeyChar == 0x2d)//负号
            {
                e.Handled = false;
                return;
            }
            if (e.KeyChar == 0x2E)
            {
                if (this.Text.Length == 0//第一位不能是小数点
                    || DecimalPlaces == 0//属性设置了没有小数
                    || this.Text.IndexOf('.') >= 0//只能输入一个小数点
                    )
                {
                    e.Handled = true;//第一位不得输入小数点
                    return;
                }
                else
                {
                    e.Handled = false;//第一位不得输入小数点
                    return;
                }
            }
            e.Handled = true;
            return;
        }

        protected override void WndProc(ref Message m)
        {
            if (!(m.Msg == (int)WindowsMsg.WM_RBUTTONDOWN ||
                m.Msg == (int)WindowsMsg.WM_PASTE)
                )
            {
                base.WndProc(ref m);
            }
        }

        public double GetDoubleValue()
        {
            return Convert.ToDouble(this.Text);
        }
        public float GetFloatValue()
        {
            return Convert.ToSingle(this.Text);
        }
        public int GetInt32Value()
        {
            return Convert.ToInt32(GetDoubleValue());
        }
    }
}
